unit UMain;

interface

uses
  Classes, SysUtils, QForms, QDialogs, QMenus, QTypes, QStdCtrls, QControls, QExtCtrls,
  QrsXmlData, QrsBinData, QrsStorage, QrsIniData, QrsUtils;

type
  TFormMain = class(TForm)
    MainMenu: TMainMenu;
    miFile: TMenuItem;
    miSep: TMenuItem;
    miExit: TMenuItem;
    miHelp: TMenuItem;
    miAbout: TMenuItem;
    meInfo: TMemo;
    pnlTop: TPanel;
    btnReadIni: TButton;
    btnReadBin: TButton;
    btnReadXML: TButton;
    btnSaveIni: TButton;
    btnSaveBin: TButton;
    btnSaveXML: TButton;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    Storage: TrsStorage;
    IniData: TrsIniData;
    BinData: TrsBinData;
    XMLData: TrsXMLData;
    procedure miExitClick(Sender: TObject);
    procedure miAboutClick(Sender: TObject);
    procedure btnReadIniClick(Sender: TObject);
    procedure btnReadBinClick(Sender: TObject);
    procedure btnReadXMLClick(Sender: TObject);
    procedure btnSaveIniClick(Sender: TObject);
    procedure btnSaveBinClick(Sender: TObject);
    procedure btnSaveXMLClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure LoadDataFromFile(const SourceName: string);
    procedure SaveDataToFile(const SourceName: string);
  end;

var
  FormMain: TFormMain;

implementation


{$R *.XFM}


procedure TFormMain.miExitClick(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.miAboutClick(Sender: TObject);
begin
  rslAbout;
end;

procedure TFormMain.btnReadIniClick(Sender: TObject);
begin
  LoadDataFromFile('ini');
end;

procedure TFormMain.btnReadBinClick(Sender: TObject);
begin
  LoadDataFromFile('bin');
end;

procedure TFormMain.btnReadXMLClick(Sender: TObject);
begin
  LoadDataFromFile('xml');
end;

procedure TFormMain.LoadDataFromFile(const SourceName: string);
begin
  with OpenDlg do begin
    DefaultExt := '*.'+SourceName;
    FileName := DefaultExt;
    InitialDir := ExtractFilePath(ParamStr(0));
    Title := 'Open '+SourceName+' file';
    Filter := 'any file|*.*|'+SourceName+' file|*.'+SourceName;
    FilterIndex := 2;
    if Execute then begin
      Storage.Clear;
      if SourceName='ini' then begin
        IniData.FileName := FileName;
        Storage.StorageData := IniData;
      end else
      if SourceName='bin' then begin
        BinData.FileName := FileName;
        Storage.StorageData := BinData;
      end else
      if SourceName='xml' then begin
        XmlData.FileName := FileName;
        Storage.StorageData := XmlData;
      end;
      Storage.Load;
      meInfo.Lines.Add('Load: '+FileName);
    end;
  end;
end;

procedure TFormMain.btnSaveIniClick(Sender: TObject);
begin
  SaveDataToFile('ini');
end;

procedure TFormMain.btnSaveBinClick(Sender: TObject);
begin
  SaveDataToFile('bin');
end;

procedure TFormMain.btnSaveXMLClick(Sender: TObject);
begin
  SaveDataToFile('xml');
end;

procedure TFormMain.SaveDataToFile(const SourceName: string);
begin
  with SaveDlg do begin
    DefaultExt := '*.'+SourceName;
    FileName := DefaultExt;
    InitialDir := ExtractFilePath(ParamStr(0));
    Title := 'Save data to '+SourceName+' file';
    Filter := 'any file|*.*|'+SourceName+' file|*.'+SourceName;
    FilterIndex := 2;
    if Execute then begin
      if SourceName='ini' then begin
        IniData.FileName := FileName;
        Storage.StorageData := IniData;
      end else
      if SourceName='bin' then begin
        BinData.FileName := FileName;
        Storage.StorageData := BinData;
      end else
      if SourceName='xml' then begin
        XmlData.FileName := FileName;
        Storage.StorageData := XmlData;
      end;
      Storage.Save;
      meInfo.Lines.Add('Save: '+FileName);
     end;
  end;
end;

end.
